@@ -1,4 +1,4 @@
-#include "../common/misc/locales.h"
+#include "../helper/locales.h"
 
 #include <gammu.h>
 #include <string.h>
@@ -9,9 +9,13 @@
 #include "memory.h"
 #include "message.h"
 #include "common.h"
-#include "formats.h"
+#include "../helper/formats.h"
+
+#include "../helper/message-display.h"
+#include "../helper/memory-display.h"
+#include "../helper/printing.h"
+#include "../helper/string.h"
 
-#ifdef GSM_ENABLE_BACKUP
 void SaveFile(int argc, char *argv[])
 {
 	GSM_Error error;
@@ -19,13 +23,13 @@ void SaveFile(int argc, char *argv[])
 	int			i;
 	size_t j;
 	FILE			*file;
-	unsigned char		Buffer[10000];
+	unsigned char		Buffer[100000];
 	GSM_MemoryEntry		*pbk;
 
 	if (strcasecmp(argv[2],"CALENDAR") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
-			exit(-1);
+			Terminate(3);
 		}
 		error=GSM_ReadBackupFile(argv[4],&Backup,GSM_GuessBackupFormat(argv[4], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
@@ -34,17 +38,19 @@ void SaveFile(int argc, char *argv[])
 			if (i == atoi(argv[5])-1) break;
 			i++;
 		}
-		if (i != atoi(argv[5])-1) {
+		if (i != atoi(argv[5])-1 || Backup.Calendar[i] == NULL) {
 			printf("%s\n", _("Calendar note not found in file"));
-			exit(-1);
+			GSM_FreeBackup(&Backup);
+			Terminate(3);
 		}
 		j = 0;
 		error = GSM_EncodeVCALENDAR(Buffer, sizeof(Buffer), &j, Backup.Calendar[i],true,Nokia_VCalendar);
+		GSM_FreeBackup(&Backup);
 		Print_Error(error);
 	} else if (strcasecmp(argv[2],"BOOKMARK") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
-			exit(-1);
+			Terminate(3);
 		}
 		error=GSM_ReadBackupFile(argv[4],&Backup,GSM_GuessBackupFormat(argv[4], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
@@ -53,16 +59,19 @@ void SaveFile(int argc, char *argv[])
 			if (i == atoi(argv[5])-1) break;
 			i++;
 		}
-		if (i != atoi(argv[5])-1) {
+		if (i != atoi(argv[5])-1 || Backup.WAPBookmark[i] == NULL) {
 			printf("%s\n", _("WAP bookmark not found in file"));
-			exit(-1);
+			GSM_FreeBackup(&Backup);
+			Terminate(3);
 		}
 		j = 0;
-		GSM_EncodeURLFile(Buffer, &j, Backup.WAPBookmark[i]);
+		error = GSM_EncodeURLFile(Buffer, &j, Backup.WAPBookmark[i]);
+		GSM_FreeBackup(&Backup);
+		Print_Error(error);
 	} else if (strcasecmp(argv[2],"NOTE") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
-			exit(-1);
+			Terminate(3);
 		}
 		error=GSM_ReadBackupFile(argv[4],&Backup,GSM_GuessBackupFormat(argv[4], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
@@ -71,17 +80,19 @@ void SaveFile(int argc, char *argv[])
 			if (i == atoi(argv[5])-1) break;
 			i++;
 		}
-		if (i != atoi(argv[5])-1) {
+		if (i != atoi(argv[5])-1 || Backup.Note[i] == NULL) {
 			printf("%s\n", _("Note not found in file"));
-			exit(-1);
+			GSM_FreeBackup(&Backup);
+			Terminate(3);
 		}
 		j = 0;
 		error = GSM_EncodeVNTFile(Buffer, sizeof(Buffer), &j, Backup.Note[i]);
+		GSM_FreeBackup(&Backup);
 		Print_Error(error);
 	} else if (strcasecmp(argv[2],"TODO") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
-			exit(-1);
+			Terminate(3);
 		}
 		error=GSM_ReadBackupFile(argv[4],&Backup,GSM_GuessBackupFormat(argv[4], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
@@ -90,17 +101,19 @@ void SaveFile(int argc, char *argv[])
 			if (i == atoi(argv[5])-1) break;
 			i++;
 		}
-		if (i != atoi(argv[5])-1) {
-			printf("%s\n", _("ToDo note not found in file"));
-			exit(-1);
+		if (i != atoi(argv[5])-1 || Backup.ToDo[i] == NULL) {
+			printf("%s\n", _("Todo note not found in file"));
+			GSM_FreeBackup(&Backup);
+			Terminate(3);
 		}
 		j = 0;
 		error = GSM_EncodeVTODO(Buffer, sizeof(Buffer), &j, Backup.ToDo[i], true, Nokia_VToDo);
+		GSM_FreeBackup(&Backup);
 		Print_Error(error);
 	} else if (strcasecmp(argv[2],"VCARD10") == 0 || strcasecmp(argv[2],"VCARD21") == 0) {
 		if (argc<6) {
 			printf("%s\n", _("Where is backup filename and location and memory type?"));
-			exit(-1);
+			Terminate(3);
 		}
 		error=GSM_ReadBackupFile(argv[4],&Backup,GSM_GuessBackupFormat(argv[4], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
@@ -110,9 +123,10 @@ void SaveFile(int argc, char *argv[])
 				if (i == atoi(argv[6])-1) break;
 				i++;
 			}
-			if (i != atoi(argv[6])-1) {
+			if (i != atoi(argv[6])-1 || Backup.SIMPhonebook[i] == NULL) {
 				printf("%s\n", _("Phonebook entry not found in file"));
-				exit(-1);
+				GSM_FreeBackup(&Backup);
+				Terminate(3);
 			}
 			pbk = Backup.SIMPhonebook[i];
 		} else if (strcasecmp(argv[5],"ME") == 0) {
@@ -120,32 +134,36 @@ void SaveFile(int argc, char *argv[])
 				if (i == atoi(argv[6])-1) break;
 				i++;
 			}
-			if (i != atoi(argv[6])-1) {
+			if (i != atoi(argv[6])-1 || Backup.PhonePhonebook[i] == NULL) {
 				printf("%s\n", _("Phonebook entry not found in file"));
-				exit(-1);
+				GSM_FreeBackup(&Backup);
+				Terminate(3);
 			}
 			pbk = Backup.PhonePhonebook[i];
 		} else {
 			printf(_("Unknown memory type: \"%s\"\n"),argv[5]);
-			exit(-1);
+			GSM_FreeBackup(&Backup);
+			Terminate(3);
 		}
 		j = 0;
 		if (strcasecmp(argv[2],"VCARD10") == 0) {
-			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard10);
+			error = GSM_EncodeVCARD(GSM_GetDebug(gsm), Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard10);
+			GSM_FreeBackup(&Backup);
 			Print_Error(error);
 		} else {
-			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard21);
+			error = GSM_EncodeVCARD(GSM_GetDebug(gsm), Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard21);
+			GSM_FreeBackup(&Backup);
 			Print_Error(error);
 		}
 	} else {
 		printf(_("Unknown backup format: \"%s\"\n"), argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	file = fopen(argv[3],"wb");
 	if (file == NULL) {
 		printf_err(_("Error while opening file for writing!\n"));
-		exit(-1);
+		Terminate(3);
 	}
 	if (j != fwrite(Buffer,1,j,file)) {
 		printf_err(_("Error while writing file!\n"));
@@ -155,7 +173,7 @@ void SaveFile(int argc, char *argv[])
 
 void DoBackup(int argc, char *argv[])
 {
-	GSM_Error error;
+	GSM_Error error = ERR_NONE;
 	int			i, used;
 	GSM_MemoryStatus	MemStatus;
 	GSM_ToDoEntry		ToDo;
@@ -176,7 +194,7 @@ void DoBackup(int argc, char *argv[])
  	GSM_FMStation		FMStation;
  	GSM_GPRSAccessPoint	GPRSPoint;
 	bool			DoBackupPart;
-	bool			UseNext;
+	bool			UseNext = false;
 	char buffer[GSM_MAX_INFO_LENGTH];
 
 	if (argc == 4 && strcasecmp(argv[3],"-yes") == 0) always_answer_yes = true;
@@ -260,6 +278,7 @@ void DoBackup(int argc, char *argv[])
 		if (UseNext) {
 			used 		= 0;
 			fprintf(stderr, LISTFORMAT, _("Reading"));
+			error = ERR_NONE;
 			while (error == ERR_NONE) {
 				if (used < GSM_BACKUP_MAX_PHONEPHONEBOOK) {
 					Backup.PhonePhonebook[used] = malloc(sizeof(GSM_MemoryEntry));
@@ -277,10 +296,11 @@ void DoBackup(int argc, char *argv[])
 				fprintf(stderr, "*");
 				if (gshutdown) {
 					GSM_Terminate();
-					exit(0);
+					Terminate(0);
 				}
 			}
 			fprintf(stderr, "\n");
+			fflush(stderr);
 		} else {
 			Pbk.MemoryType  = MEM_ME;
 			i		= 1;
@@ -304,11 +324,13 @@ void DoBackup(int argc, char *argv[])
 					used++;
 				}
 				fprintf(stderr, "\r   ");
-				fprintf(stderr, _("Reading: %i percent"), used*100/MemStatus.MemoryUsed);
+				fprintf(stderr, "%s ", _("Reading:"));
+				fprintf(stderr, _("%i percent"),
+					used * 100 / MemStatus.MemoryUsed);
 				i++;
 				if (gshutdown) {
 					GSM_Terminate();
-					exit(0);
+					Terminate(0);
 				}
 			}
 			fprintf(stderr, "\n");
@@ -346,11 +368,13 @@ void DoBackup(int argc, char *argv[])
 				used++;
 			}
 			fprintf(stderr, "\r   ");
-			fprintf(stderr, _("Reading: %i percent"), used*100/MemStatus.MemoryUsed);
+			fprintf(stderr, "%s ", _("Reading:"));
+			fprintf(stderr, _("%i percent"),
+				used * 100 / MemStatus.MemoryUsed);
 			i++;
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -358,6 +382,7 @@ void DoBackup(int argc, char *argv[])
 	DoBackupPart = false;
 	if (Info.Calendar) {
 		printf("%s\n", _("Checking phone calendar"));
+		Calendar.Location = 0;
 		error=GSM_GetNextCalendar(gsm,&Calendar,true);
 		if (error==ERR_NONE) {
 			if (answer_yes("   %s", _("Backup phone calendar notes?"))) DoBackupPart = true;
@@ -383,21 +408,23 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.ToDo) {
-		printf("%s\n", _("Checking phone ToDo"));
+		printf("%s\n", _("Checking phone todos"));
 		error=GSM_GetToDoStatus(gsm,&ToDoStatus);
 		if (error == ERR_NONE && ToDoStatus.Used != 0) {
-			if (answer_yes("   %s", _("Backup phone ToDo?"))) DoBackupPart = true;
+			if (answer_yes("   %s", _("Backup phone todos?"))) DoBackupPart = true;
 		}
 	}
 	if (DoBackupPart) {
 		used = 0;
+		ToDo.Location = 0;
 		error=GSM_GetNextToDo(gsm,&ToDo,true);
 		while (error == ERR_NONE) {
 			if (used < GSM_MAXCALENDARTODONOTES) {
@@ -414,10 +441,12 @@ void DoBackup(int argc, char *argv[])
 			used ++;
 			error=GSM_GetNextToDo(gsm,&ToDo,false);
 			fprintf(stderr, "\r   ");
-			fprintf(stderr, _("Reading: %i percent"), used*100/ToDoStatus.Used);
+			fprintf(stderr, "%s ", _("Reading:"));
+			fprintf(stderr, _("%i percent"),
+				used * 100 / ToDoStatus.Used);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -425,6 +454,7 @@ void DoBackup(int argc, char *argv[])
 	DoBackupPart = false;
 	if (Info.Note) {
 		printf("%s\n", _("Checking phone notes"));
+		Note.Location = 0;
 		error=GSM_GetNextNote(gsm,&Note,true);
 		if (error==ERR_NONE) {
 			if (answer_yes("   %s", _("Backup phone notes?"))) DoBackupPart = true;
@@ -450,10 +480,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.CallerLogos) {
@@ -487,10 +518,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.SMSC) {
@@ -519,6 +551,7 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.StartupLogo) {
@@ -567,6 +600,7 @@ void DoBackup(int argc, char *argv[])
 	if (DoBackupPart) {
 		used = 0;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE) {
 			if (used < GSM_BACKUP_MAX_WAPBOOKMARK) {
 				Backup.WAPBookmark[used] = malloc(sizeof(GSM_WAPBookmark));
@@ -585,10 +619,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.WAPSettings) {
@@ -602,6 +637,7 @@ void DoBackup(int argc, char *argv[])
 	if (DoBackupPart) {
 		used = 0;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE) {
 			if (used < GSM_BACKUP_MAX_WAPSETTINGS) {
 				Backup.WAPSettings[used] = malloc(sizeof(GSM_MultiWAPSettings));
@@ -620,10 +656,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.MMSSettings) {
@@ -637,6 +674,7 @@ void DoBackup(int argc, char *argv[])
 	if (DoBackupPart) {
 		used = 0;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE) {
 			if (used < GSM_BACKUP_MAX_MMSSETTINGS) {
 				Backup.MMSSettings[used] = malloc(sizeof(GSM_MultiWAPSettings));
@@ -655,10 +693,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.ChatSettings) {
@@ -672,6 +711,7 @@ void DoBackup(int argc, char *argv[])
 	if (DoBackupPart) {
 		used = 0;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE) {
 			if (used < GSM_BACKUP_MAX_CHATSETTINGS) {
 				Backup.ChatSettings[used] = malloc(sizeof(GSM_ChatSettings));
@@ -690,10 +730,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.SyncMLSettings) {
@@ -725,10 +766,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.Ringtone) {
@@ -744,6 +786,7 @@ void DoBackup(int argc, char *argv[])
 		used 	= 0;
 		i	= 1;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE || error == ERR_EMPTY) {
 			if (error == ERR_NONE) {
 				if (used < GSM_BACKUP_MAX_RINGTONES) {
@@ -766,10 +809,11 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
 	if (Info.Profiles) {
@@ -802,6 +846,7 @@ void DoBackup(int argc, char *argv[])
 			fprintf(stderr, "*");
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	DoBackupPart = false;
  	if (Info.FMStation) {
@@ -816,6 +861,7 @@ void DoBackup(int argc, char *argv[])
 		used	= 0;
 		i	= 1;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE || error == ERR_EMPTY) {
 			error = GSM_GetFMStation(gsm,&FMStation);
 			if (error == ERR_NONE) {
@@ -837,6 +883,7 @@ void DoBackup(int argc, char *argv[])
  			fprintf(stderr, "*");
  		}
  		fprintf(stderr, "\n");
+		fflush(stderr);
  	}
 	DoBackupPart = false;
  	if (Info.GPRSPoint) {
@@ -851,6 +898,7 @@ void DoBackup(int argc, char *argv[])
 		used	= 0;
 		i	= 1;
 		fprintf(stderr, LISTFORMAT, _("Reading"));
+		error = ERR_NONE;
 		while (error == ERR_NONE || error == ERR_EMPTY) {
 			error = GSM_GetGPRSAccessPoint(gsm,&GPRSPoint);
  			if (error == ERR_NONE) {
@@ -872,6 +920,7 @@ void DoBackup(int argc, char *argv[])
  			fprintf(stderr, "*");
  		}
  		fprintf(stderr, "\n");
+		fflush(stderr);
  	}
 
 	GSM_Terminate();
@@ -920,10 +969,8 @@ void Restore(int argc, char *argv[])
 	if (argc == 4 && strcasecmp(argv[3],"-yes") == 0) always_answer_yes = true;
 
 	if (Backup.MD5Calculated[0]!=0) {
-		dbgprintf("\"%s\"\n",Backup.MD5Original);
-		dbgprintf("\"%s\"\n",Backup.MD5Calculated);
 		if (strcmp(Backup.MD5Original,Backup.MD5Calculated)) {
-			if (!answer_yes(_("Checksum in backup file do not match. Continue?"))) return;
+			if (!answer_yes(_("Checksum in backup file do not match (original: %s, new: %s). Continue?"), Backup.MD5Original, Backup.MD5Calculated)) return;
 		}
 	}
 
@@ -946,10 +993,13 @@ void Restore(int argc, char *argv[])
 		for (i=0;i<max;i++) {
 			error=GSM_SetBitmap(gsm,Backup.CallerLogos[i]);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -975,7 +1025,6 @@ void Restore(int argc, char *argv[])
 			if (used<max && Backup.PhonePhonebook[used]->Location == Pbk.Location) {
 				Pbk = *Backup.PhonePhonebook[used];
 				used++;
-				dbgprintf("Location %i\n",Pbk.Location);
 				if (Pbk.EntriesNum != 0) error=GSM_SetMemory(gsm, &Pbk);
 				if (error == ERR_PERMISSION && GSM_IsPhoneFeatureAvailable(GSM_GetModelInfo(gsm), F_6230iCALLER)) {
 					error=GSM_DeleteMemory(gsm, &Pbk);
@@ -997,7 +1046,7 @@ void Restore(int argc, char *argv[])
 							printf("%20s\n    ", " ");
 							First = false;
 						}
-						PrintMemorySubEntry(&Pbk.Entries[j]);
+						PrintMemorySubEntry(&Pbk.Entries[j], gsm);
 						printf("    %s\n", GSM_ErrorString(Pbk.Entries[j].AddError));
 					}
 				}
@@ -1008,13 +1057,14 @@ void Restore(int argc, char *argv[])
 					error=GSM_DeleteMemory(gsm, &Pbk);
 			}
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),
-					13,
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
 					(i + 1) * 100 / (MemStatus.MemoryUsed + MemStatus.MemoryFree)
 					);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1040,7 +1090,6 @@ void Restore(int argc, char *argv[])
 			if (used<max && Backup.SIMPhonebook[used]->Location == Pbk.Location) {
 				Pbk = *Backup.SIMPhonebook[used];
 				used++;
-				dbgprintf("Location %i\n",Pbk.Location);
 				if (Pbk.EntriesNum != 0) {
 					error=GSM_SetMemory(gsm, &Pbk);
 					if (error==ERR_NONE) {
@@ -1053,7 +1102,7 @@ void Restore(int argc, char *argv[])
 								printf("%20s\n    ", " ");
 								First = false;
 							}
-							PrintMemorySubEntry(&Pbk.Entries[j]);
+							PrintMemorySubEntry(&Pbk.Entries[j], gsm);
 							printf("    %s\n",GSM_ErrorString(Pbk.Entries[j].AddError));
 						}
 					}
@@ -1061,10 +1110,13 @@ void Restore(int argc, char *argv[])
 			}
 			if (Pbk.EntriesNum == 0) error=GSM_DeleteMemory(gsm, &Pbk);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/(MemStatus.MemoryUsed+MemStatus.MemoryFree));
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / (MemStatus.MemoryUsed + MemStatus.MemoryFree));
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1080,6 +1132,7 @@ void Restore(int argc, char *argv[])
 	}
 	DoRestore = false;
 	if (Backup.Calendar[0] != NULL) {
+		Calendar.Location = 0;
 		/* N6110 doesn't support getting calendar status */
 		error = GSM_GetNextCalendar(gsm,&Calendar,true);
 		if (error == ERR_NONE || error == ERR_INVALIDLOCATION || error == ERR_EMPTY) {
@@ -1093,7 +1146,7 @@ void Restore(int argc, char *argv[])
 		}
 	}
 	if (DoRestore) {
-		fprintf(stderr, _("Deleting old notes: "));
+		fprintf(stderr, "%s ", _("Deleting old notes:"));
 		error = GSM_DeleteAllCalendar(gsm);
 		if (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED) {
  			while (1) {
@@ -1115,10 +1168,13 @@ void Restore(int argc, char *argv[])
 			Calendar = *Backup.Calendar[i];
 			error=GSM_AddCalendar(gsm,&Calendar);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1132,7 +1188,7 @@ void Restore(int argc, char *argv[])
 			while (Backup.ToDo[max]!=NULL) max++;
 			fprintf(stderr, _("%i entries in backup file\n"),max);
 
-			if (answer_yes(_("Restore phone ToDo?"))) DoRestore = true;
+			if (answer_yes(_("Restore phone todo?"))) DoRestore = true;
 		}
 	}
 	if (DoRestore) {
@@ -1140,7 +1196,7 @@ void Restore(int argc, char *argv[])
 		error = GSM_SetToDo(gsm,&ToDo);
 	}
 	if (DoRestore && (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED)) {
-		fprintf(stderr, _("Deleting old ToDo: "));
+		fprintf(stderr, "%s ", _("Deleting old todos:"));
 		error=GSM_DeleteAllToDo(gsm);
 		if (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED) {
 			while (1) {
@@ -1161,10 +1217,13 @@ void Restore(int argc, char *argv[])
 			ToDo.Location 	= 0;
 			error=GSM_AddToDo(gsm,&ToDo);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1186,10 +1245,13 @@ void Restore(int argc, char *argv[])
 				Print_Error(error);
 			}
 			error = GSM_GetNextToDo(gsm,&ToDo,false);
-			fprintf(stderr, _("%cCleaning: %i percent"),13,used*100/ToDoStatus.Used);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				used * 100 / ToDoStatus.Used);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1199,10 +1261,13 @@ void Restore(int argc, char *argv[])
 			ToDo  = *Backup.ToDo[i];
 			error = GSM_SetToDo(gsm,&ToDo);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1216,11 +1281,11 @@ void Restore(int argc, char *argv[])
 			while (Backup.Note[max]!=NULL) max++;
 			fprintf(stderr, _("%i entries in backup file\n"),max);
 
-			if (answer_yes(_("Restore phone Notes?"))) DoRestore = true;
+			if (answer_yes(_("Restore phone notes?"))) DoRestore = true;
 		}
 	}
 	if (DoRestore) {
-		fprintf(stderr, _("Deleting old Notes: "));
+		fprintf(stderr, "%s ", _("Deleting old notes:"));
 		while (1) {
 			error = GSM_GetNextNote(gsm,&Note,true);
 			if (error != ERR_NONE) break;
@@ -1235,10 +1300,13 @@ void Restore(int argc, char *argv[])
 			Note.Location 	= 0;
 			error=GSM_AddNote(gsm,&Note);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1250,10 +1318,13 @@ void Restore(int argc, char *argv[])
 		for (i=0;i<max;i++) {
 			error=GSM_SetSMSC(gsm,Backup.SMSC[i]);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1275,7 +1346,7 @@ void Restore(int argc, char *argv[])
 		}
 	}
 	if (DoRestore) {
-		fprintf(stderr, _("Deleting old bookmarks: "));
+		fprintf(stderr, "%s ", _("Deleting old bookmarks:"));
 		/* One thing to explain: DCT4 phones seems to have bug here.
 		 * When delete for example first bookmark, phone change
 		 * numeration for getting frame, not for deleting. So, we try to
@@ -1296,10 +1367,13 @@ void Restore(int argc, char *argv[])
 			Bookmark.Location = 0;
 			error=GSM_SetWAPBookmark(gsm,&Bookmark);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1318,10 +1392,13 @@ void Restore(int argc, char *argv[])
 		for (i=0;i<max;i++) {
 			error=GSM_SetWAPSettings(gsm,Backup.WAPSettings[i]);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1340,10 +1417,13 @@ void Restore(int argc, char *argv[])
 		for (i=0;i<max;i++) {
 			error=GSM_SetMMSSettings(gsm,Backup.MMSSettings[i]);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1373,10 +1453,13 @@ void Restore(int argc, char *argv[])
 			Print_Error(error);
 			error=GSM_SetRingtone(gsm,&Ringtone,&i);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1399,7 +1482,7 @@ void Restore(int argc, char *argv[])
 			Print_Error(error);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1413,7 +1496,7 @@ void Restore(int argc, char *argv[])
 		}
 	}
 	if (DoRestore) {
-		fprintf(stderr, _("Deleting old FM stations: "));
+		fprintf(stderr, "%s ", _("Deleting old FM stations:"));
 		error=GSM_ClearFMStations(gsm);
 		Print_Error(error);
 		fprintf(stderr, "%s\n", _("Done"));
@@ -1423,10 +1506,13 @@ void Restore(int argc, char *argv[])
 			FMStation = *Backup.FMStation[i];
 			error=GSM_SetFMStation(gsm,&FMStation);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1445,10 +1531,13 @@ void Restore(int argc, char *argv[])
 		for (i=0;i<max;i++) {
 			error=GSM_SetGPRSAccessPoint(gsm,Backup.GPRSPoint[i]);
 			Print_Error(error);
-			fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+			fprintf(stderr, "\r");
+			fprintf(stderr, "%s ", _("Writing:"));
+			fprintf(stderr, _("%i percent"),
+				(i + 1) * 100 / max);
 			if (gshutdown) {
 				GSM_Terminate();
-				exit(0);
+				Terminate(0);
 			}
 		}
 		fprintf(stderr, "\n");
@@ -1500,10 +1589,13 @@ void AddNew(int argc, char *argv[])
 					Pbk.MemoryType 	= MEM_ME;
 					error=GSM_AddMemory(gsm, &Pbk);
 					Print_Error(error);
-					fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+					fprintf(stderr, "\r");
+					fprintf(stderr, "%s ", _("Writing:"));
+					fprintf(stderr, _("%i percent"),
+						(i + 1) * 100 / max);
 					if (gshutdown) {
 						GSM_Terminate();
-						exit(0);
+						Terminate(0);
 					}
 				}
 				fprintf(stderr, "\n");
@@ -1525,10 +1617,13 @@ void AddNew(int argc, char *argv[])
 					Pbk.MemoryType 	= MEM_SM;
 					error=GSM_AddMemory(gsm, &Pbk);
 					Print_Error(error);
-					fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+					fprintf(stderr, "\r");
+					fprintf(stderr, "%s ", _("Writing:"));
+					fprintf(stderr, _("%i percent"),
+						(i + 1) * 100 / max);
 					if (gshutdown) {
 						GSM_Terminate();
-						exit(0);
+						Terminate(0);
 					}
 				}
 				fprintf(stderr, "\n");
@@ -1554,10 +1649,13 @@ void AddNew(int argc, char *argv[])
 					Calendar = *Backup.Calendar[i];
 					error=GSM_AddCalendar(gsm,&Calendar);
 					Print_Error(error);
-					fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+					fprintf(stderr, "\r");
+					fprintf(stderr, "%s ", _("Writing:"));
+					fprintf(stderr, _("%i percent"),
+						(i + 1) * 100 / max);
 					if (gshutdown) {
 						GSM_Terminate();
-						exit(0);
+						Terminate(0);
 					}
 				}
 				fprintf(stderr, "\n");
@@ -1575,10 +1673,13 @@ void AddNew(int argc, char *argv[])
 					ToDo  = *Backup.ToDo[i];
 					error = GSM_AddToDo(gsm,&ToDo);
 					Print_Error(error);
-					fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+					fprintf(stderr, "\r");
+					fprintf(stderr, "%s ", _("Writing:"));
+					fprintf(stderr, _("%i percent"),
+						(i + 1) * 100 / max);
 					if (gshutdown) {
 						GSM_Terminate();
-						exit(0);
+						Terminate(0);
 					}
 				}
 				fprintf(stderr, "\n");
@@ -1597,10 +1698,13 @@ void AddNew(int argc, char *argv[])
 					Bookmark.Location = 0;
 					error=GSM_SetWAPBookmark(gsm,&Bookmark);
 					Print_Error(error);
-					fprintf(stderr, _("%cWriting: %i percent"),13,(i+1)*100/max);
+					fprintf(stderr, "\r");
+					fprintf(stderr, "%s ", _("Writing:"));
+					fprintf(stderr, _("%i percent"),
+						(i + 1) * 100 / max);
 					if (gshutdown) {
 						GSM_Terminate();
-						exit(0);
+						Terminate(0);
 					}
 				}
 				fprintf(stderr, "\n");
@@ -1612,162 +1716,6 @@ void AddNew(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-void BackupSMS(int argc UNUSED, char *argv[])
-{
-	GSM_Error error;
-	GSM_SMS_Backup		Backup;
-	GSM_MultiSMSMessage 	sms;
-	GSM_SMSFolders		folders;
-	bool			BackupFromFolder[GSM_MAX_SMS_FOLDERS];
-	bool			start = true;
-	bool			DeleteAfter;
-	int			j, smsnum = 0;
-	char			buffer[200];
-
-	GSM_Init(true);
-
-	error=GSM_GetSMSFolders(gsm, &folders);
-	Print_Error(error);
-
-	DeleteAfter=answer_yes(_("Delete each sms after backup?"));
-
-	for (j=0;j<folders.Number;j++) {
-		BackupFromFolder[j] = false;
-		if (folders.Folder[j].Memory == MEM_SM) strcat(buffer," (SIM)");
-		if (answer_yes(_("Backup sms from folder \"%s\"%s?"),
-				DecodeUnicodeConsole(folders.Folder[j].Name),
-				folders.Folder[j].Memory == MEM_SM ? _(" (SIM)") : ""
-				))
-			BackupFromFolder[j] = true;
-	}
-
-	while (error == ERR_NONE) {
-		sms.SMS[0].Folder=0x00;
-		error=GSM_GetNextSMS(gsm, &sms, start);
-		switch (error) {
-		case ERR_EMPTY:
-			break;
-		default:
-			Print_Error(error);
-			for (j=0;j<sms.Number;j++) {
-				if (BackupFromFolder[sms.SMS[j].Folder-1]) {
-					switch (sms.SMS[j].PDU) {
-					case SMS_Status_Report:
-						break;
-					case SMS_Submit:
-					case SMS_Deliver:
-						if (sms.SMS[j].Length == 0) break;
-						if (smsnum < GSM_BACKUP_MAX_SMS) {
-							Backup.SMS[smsnum] = malloc(sizeof(GSM_SMSMessage));
-						        if (Backup.SMS[smsnum] == NULL) Print_Error(ERR_MOREMEMORY);
-							Backup.SMS[smsnum + 1] = NULL;
-						} else {
-							printf(_("   Increase %s\n") , "GSM_BACKUP_MAX_SMS");
-							GSM_Terminate();
-							exit(-1);
-						}
-						*Backup.SMS[smsnum] = sms.SMS[j];
-						smsnum++;
-						break;
-					}
-				}
-			}
-		}
-		start=false;
-	}
-
-	error = GSM_AddSMSBackupFile(argv[2],&Backup);
-	Print_Error(error);
-
-	if (DeleteAfter) {
-		for (j=0;j<smsnum;j++) {
-			Backup.SMS[j]->Folder = 0;
-			error=GSM_DeleteSMS(gsm, Backup.SMS[j]);
-			Print_Error(error);
-			fprintf(stderr, _("%cDeleting: %i percent"),13,(j+1)*100/smsnum);
-		}
-	}
-
-	GSM_Terminate();
-}
-
-void AddSMS(int argc UNUSED, char *argv[])
-{
-	GSM_Error error;
-	GSM_MultiSMSMessage 	SMS;
-	GSM_SMS_Backup		Backup;
-	int			smsnum = 0;
-	int			folder;
-
-	folder = atoi(argv[2]);
-
-	error = GSM_ReadSMSBackupFile(argv[3], &Backup);
-	Print_Error(error);
-
-	GSM_Init(true);
-
-	while (Backup.SMS[smsnum] != NULL) {
-		Backup.SMS[smsnum]->Folder = folder;
-		Backup.SMS[smsnum]->SMSC.Location = 1;
-		SMS.Number = 1;
-		SMS.SMS[0] = *Backup.SMS[smsnum];
-		DisplayMultiSMSInfo(&SMS,false,false,NULL);
-		if (answer_yes(_("Restore sms?"))) {
-			error=GSM_AddSMS(gsm, Backup.SMS[smsnum]);
-			Print_Error(error);
-		}
-		smsnum++;
-	}
-
-	/* We don't need this anymore */
-	GSM_FreeSMSBackup(&Backup);
-
-	GSM_Terminate();
-}
-
-void RestoreSMS(int argc UNUSED, char *argv[])
-{
-	GSM_Error error;
-	GSM_MultiSMSMessage 	SMS;
-	GSM_SMS_Backup		Backup;
-	GSM_SMSFolders		folders;
-	int			smsnum = 0;
-	bool			restore8bit;
-
-	error = GSM_ReadSMSBackupFile(argv[2], &Backup);
-	Print_Error(error);
-
-	restore8bit = answer_yes(_("Do you want to restore binary SMS?"));
-
-	GSM_Init(true);
-
-	error = GSM_GetSMSFolders(gsm, &folders);
-	Print_Error(error);
-
-	while (Backup.SMS[smsnum] != NULL) {
-		if (restore8bit || Backup.SMS[smsnum]->Coding != SMS_Coding_8bit) {
-			SMS.Number = 1;
-			memcpy(&SMS.SMS[0],Backup.SMS[smsnum],sizeof(GSM_SMSMessage));
-			DisplayMultiSMSInfo(&SMS,false,false,NULL);
-			if (answer_yes(_("Restore %03i sms to folder \"%s\"%s?"),
-					smsnum + 1,
-					DecodeUnicodeConsole(folders.Folder[Backup.SMS[smsnum]->Folder - 1].Name),
-					folders.Folder[Backup.SMS[smsnum]->Folder - 1].Memory == MEM_SM ? _(" (SIM)") : "")) {
-				smprintf(gsm, _("saving %i SMS\n"),smsnum);
-				error = GSM_AddSMS(gsm, Backup.SMS[smsnum]);
-				Print_Error(error);
-			}
-		}
-		smsnum++;
-	}
-
-	/* We don't need this anymore */
-	GSM_FreeSMSBackup(&Backup);
-
-	GSM_Terminate();
-}
-#endif
-
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
