@@ -3,72 +3,25 @@
 #include <signal.h>
 #include <stdlib.h>
 
-#include "../common/misc/locales.h"
+#include "../helper/locales.h"
 
 #include "common.h"
-#include "formats.h"
+#include "../helper/formats.h"
+#include "../helper/printing.h"
 
 #ifdef CURL_FOUND
 #include <curl/curl.h>
 #endif
 
-GSM_StateMachine *gsm;
+GSM_StateMachine *gsm = NULL;
+INI_Section *cfg = NULL;
 
 bool always_answer_yes = false;
 bool always_answer_no = false;
 volatile bool gshutdown = false;
-bool phonedb = false;
 bool batch = false;
 bool batchConn = false;
 
-PRINTF_STYLE(1, 2)
-int printf_err(const char *format, ...)
-{
-	va_list ap;
-	int ret;
-
-	/* l10n: Generic prefix for error messages */
-	printf("%s: ", _("Error"));
-
-	va_start(ap, format);
-	ret = vprintf(format, ap);
-	va_end(ap);
-
-	return ret;
-}
-
-PRINTF_STYLE(1, 2)
-int printf_warn(const char *format, ...)
-{
-	va_list ap;
-	int ret;
-
-	/* l10n: Generic prefix for warning messages */
-	printf("%s: ", _("Warning"));
-
-	va_start(ap, format);
-	ret = vprintf(format, ap);
-	va_end(ap);
-
-	return ret;
-}
-
-PRINTF_STYLE(1, 2)
-int printf_info(const char *format, ...)
-{
-	va_list ap;
-	int ret;
-
-	/* l10n: Generic prefix for informational messages */
-	printf("%s: ", _("Information"));
-
-	va_start(ap, format);
-	ret = vprintf(format, ap);
-	va_end(ap);
-
-	return ret;
-}
-
 void interrupt(int sign)
 {
 	signal(sign, SIG_IGN);
@@ -101,6 +54,9 @@ void PrintSecurityStatus(void)
 		case SEC_Phone:
 			printf("%s\n", _("Waiting for phone code."));
 			break;
+		case SEC_Network:
+			printf("%s\n", _("Waiting for network code."));
+			break;
 		case SEC_None:
 			printf("%s\n", _("Nothing to enter."));
 			break;
@@ -192,17 +148,27 @@ const char *GetDayName(const int day)
 	return result;
 }
 
+NORETURN
+void Terminate(int code)
+{
+	Cleanup();
+	exit(code);
+}
+
 void Cleanup(void)
 {
 	GSM_Debug_Info *di;
 
-	/* Disconnect from phone */
-	if (GSM_IsConnected(gsm)) {
-		GSM_TerminateConnection(gsm);
-	}
+	if (gsm != NULL) {
+		/* Disconnect from phone */
+		if (GSM_IsConnected(gsm)) {
+			GSM_TerminateConnection(gsm);
+		}
 
-	/* Free state machine */
-	GSM_FreeStateMachine(gsm);
+		/* Free state machine */
+		GSM_FreeStateMachine(gsm);
+	}
+	gsm = NULL;
 
 	/* Close debug output if opened */
 	di = GSM_GetGlobalDebug();
@@ -212,6 +178,9 @@ void Cleanup(void)
 	/* Free CURL memory */
 	curl_global_cleanup();
 #endif
+
+	INI_Free(cfg);
+	cfg = NULL;
 }
 
 void Print_Error(GSM_Error error)
@@ -224,9 +193,7 @@ void Print_Error(GSM_Error error)
 			PrintSecurityStatus();
 		}
 
-		Cleanup();
-
-		exit(-1);
+		Terminate(3);
 	}
 }
 
@@ -293,12 +260,6 @@ bool GSM_ReadHTTPFile(const char *url, GSM_File *file)
  */
 void GSM_Init(bool checkerror)
 {
-	GSM_File PhoneDB;
-	char model[GSM_MAX_MODEL_LENGTH];
-	char current_version[GSM_MAX_VERSION_LENGTH];
-	char url[70 + GSM_MAX_MODEL_LENGTH];
-	char latest_version[GSM_MAX_VERSION_LENGTH];
-	size_t pos = 0, oldpos = 0, i;
 	GSM_Error error;
 
 	/* No checks on existing batch mode */
@@ -316,66 +277,6 @@ void GSM_Init(bool checkerror)
 			batchConn = true;
 		}
 	}
-
-	/* No phonedb check? */
-	if (!phonedb)
-		return;
-
-	/* Get model information */
-	error = GSM_GetModel(gsm, model);
-	Print_Error(error);
-
-	/* Empty string */
-	latest_version[0] = 0;
-
-	/* Request information from phone db */
-	sprintf(url, "http://www.gammu.org/support/phones/phonedbxml.php?model=%s", model);
-	PhoneDB.Buffer = NULL;
-	PhoneDB.Used = 0;
-	if (!GSM_ReadHTTPFile(url, &PhoneDB))
-		return;
-
-	/* Parse reply */
-	while (pos < PhoneDB.Used) {
-		if (PhoneDB.Buffer[pos] != 10) {
-			pos++;
-			continue;
-		}
-		PhoneDB.Buffer[pos] = 0;
-		if (strstr(PhoneDB.Buffer + oldpos, "<firmware>") == NULL) {
-			pos++;
-			oldpos = pos;
-			continue;
-		}
-		sprintf(latest_version, "%s", strstr(PhoneDB.Buffer + oldpos, "<version>") + 9);
-		for (i = 0; i < strlen(latest_version); i++) {
-			if (latest_version[i] == '<') {
-				latest_version[i] = 0;
-				break;
-			}
-		}
-		pos++;
-		oldpos = pos;
-	}
-	free(PhoneDB.Buffer);
-
-	/* Did we find something? */
-	if (latest_version[0] == 0) {
-		return;
-	}
-
-	/* Get phone firmware version */
-	error = GSM_GetFirmware(gsm, current_version, NULL, NULL);
-	Print_Error(error);
-
-	/* Compare versions */
-	if (!GSM_IsNewerVersion(latest_version, current_version))
-		return;
-
-	/* Print information to user */
-	printf_info(_("Never version of firmware is available!\n"));
-	printf_info(_("Latest version is %s and you run %s.\n"),
-			latest_version, current_version);
 }
 
 void GSM_Terminate(void)
@@ -394,13 +295,13 @@ void GetStartStop(int *start, int *stop, int num, int argc, char *argv[])
 
 	if (argc <= num) {
 		printf_err("%s\n", _("More parameters required!"));
-		exit(-1);
+		Terminate(3);
 	}
 
 	*start = atoi(argv[num]);
 	if (*start == 0) {
 		printf_err("%s\n", _("Please enumerate locations from 1"));
-		exit(-1);
+		Terminate(3);
 	}
 
 	if (stop != NULL) {
@@ -410,7 +311,7 @@ void GetStartStop(int *start, int *stop, int num, int argc, char *argv[])
 		if (*stop == 0) {
 			printf_err("%s\n",
 				   _("Please enumerate locations from 1"));
-			exit(-1);
+			Terminate(3);
 		}
 		if (*stop < *start) {
 			printf_warn("%s\n",
@@ -448,7 +349,7 @@ bool answer_yes(const char *format, ...)
 		}
 		len = GetLine(stdin, ans, 99);
 		if (len == -1)
-			exit(-1);
+			Terminate(3);
 		/* l10n: Answer to (yes/no/ALL/ONLY/NONE) question */
 		if (!strcmp(ans, _("NONE"))) {
 			always_answer_no = true;
