@@ -2,10 +2,11 @@
 #define _gammu_common_h
 
 #include <gammu.h>
+#include "../helper/string.h"
 
 extern volatile bool gshutdown;
 extern GSM_StateMachine *gsm;
-extern bool phonedb;
+extern INI_Section *cfg;
 extern bool batch;
 extern bool batchConn;
 extern bool always_answer_yes;
@@ -33,24 +34,6 @@ void PrintSecurityStatus(void);
  */
 GSM_MemoryType MemoryTypeFromString(const char *type);
 
-/**
- * Write error to user.
- */
-PRINTF_STYLE(1, 2)
-int printf_err(const char *format, ...);
-
-/**
- * Write warning to user.
- */
-PRINTF_STYLE(1, 2)
-int printf_warn(const char *format, ...);
-
-/**
- * Write information to user.
- */
-PRINTF_STYLE(1, 2)
-int printf_info(const char *format, ...);
-
 /**
  * Interrupt handler, sets shutdown flag.
  */
@@ -71,6 +54,12 @@ bool GSM_ReadHTTPFile(const char *url, GSM_File *file);
  */
 void Cleanup(void);
 
+/**
+ * Cleanups and terminates program.
+ */
+NORETURN
+void Terminate(int code);
+
 #ifdef GSM_ENABLE_BEEP
 void GSM_PhoneBeep(void);
 #endif
