@@ -7,11 +7,14 @@
 
 #include <string.h>
 #include <signal.h>
+#include <stdlib.h>
 
-#include "../../../common/gsmstate.h"
-#include "../../../common/phone/nokia/dct3/dct3func.h"
-#include "../../../common/phone/pfunc.h"
+#include "../../../libgammu/gsmstate.h"
+#include "../../../libgammu/phone/nokia/dct3/dct3func.h"
+#include "../../../libgammu/phone/pfunc.h"
+#include "../../../libgammu/service/gsmnet.h"
 #include "../../gammu.h"
+#include "../../../helper/string.h"
 
 extern GSM_Reply_Function UserReplyFunctions3[];
 
@@ -54,7 +57,7 @@ static void CheckDCT3(void)
 	case ERR_OTHERCONNECTIONREQUIRED:
 		printf("Can't do it with current phone protocol\n");
 		GSM_TerminateConnection(gsm);
-		exit(-1);
+		Terminate(3);
 	default:
 		break;
 	}
@@ -68,7 +71,7 @@ static bool answer_yes3(const char *text)
 	while (1) {
 		printf("%s (yes/no) ? ",text);
 		len=GetLine(stdin, ans, 99);
-		if (len==-1) exit(-1);
+		if (len==-1) Terminate(3);
 		if (strcasecmp(ans, "yes") == 0) return true;
 		if (strcasecmp(ans, "no" ) == 0) return false;
 	}
@@ -504,9 +507,9 @@ static GSM_Error DCT3_ReplyGetPPS(GSM_Protocol_Message msg, GSM_StateMachine *sm
 	int i,j,z;
 
 #ifdef DEBUG
-	dbgprintf("Product Profile Settings received -");
-	for (i=0;i<4;i++) dbgprintf(" %02x",msg.Buffer[3+i]);
-	dbgprintf("\n");
+	smprintf(sm, "Product Profile Settings received -");
+	for (i=0;i<4;i++) smprintf(sm, " %02x",msg.Buffer[3+i]);
+	smprintf(sm, "\n");
 #endif
 	j=128;z=0;
 	for (i=0;i<32;i++) {
@@ -518,9 +521,9 @@ static GSM_Error DCT3_ReplyGetPPS(GSM_Protocol_Message msg, GSM_StateMachine *sm
 		} else j=j/2;
 	}
 #ifdef DEBUG
-	dbgprintf("After decoding: ");
-	for (i=0;i<32;i++) dbgprintf("%c",PPS[i]);
-	dbgprintf("\n");
+	smprintf(sm, "After decoding: ");
+	for (i=0;i<32;i++) smprintf(sm, "%c",PPS[i]);
+	smprintf(sm, "\n");
 #endif
 	return ERR_NONE;
 }
@@ -578,9 +581,9 @@ void DCT3SetPhoneMenus(int argc, char *argv[])
 	}
 
 #ifdef DEBUG
-	dbgprintf("After settings: ");
-	for (i=0;i<32;i++) dbgprintf("%c",PPS[i]);
-	dbgprintf("\n");
+	smprintf(gsm, "After settings: ");
+	for (i=0;i<32;i++) smprintf(gsm, "%c",PPS[i]);
+	smprintf(gsm, "\n");
 #endif
 
 	j=128;z=0;
@@ -737,14 +740,14 @@ static GSM_Error DCT3_ReplyDisplayOutput(GSM_Protocol_Message msg, GSM_StateMach
 
 	switch (msg.Buffer[3]) {
 	case 0x50:
-		dbgprintf("Display string received\n");
+		smprintf(sm, "Display string received\n");
 		memcpy(buf,msg.Buffer+8,msg.Buffer[7]*2);
 		buf[msg.Buffer[7]*2]   = 0;
 		buf[msg.Buffer[7]*2+1] = 0;
 		printf("X=%i, Y=%i, Text=\"%s\"\n",msg.Buffer[6],msg.Buffer[5],DecodeUnicodeString(buf));
 		return ERR_NONE;
 	case 0x54:
-		dbgprintf("Display output set\n");
+		smprintf(sm, "Display output set\n");
 		return ERR_NONE;
 	}
 	return ERR_UNKNOWNRESPONSE;
