@@ -1,6 +1,6 @@
 /* (c) 2002-2004 by Marcin Wiacek */
 
-#include "../../../common/gsmstate.h"
+#include "../../../libgammu/gsmstate.h"
 
 #ifdef GSM_ENABLE_NOKIA_DCT4
 
@@ -8,11 +8,14 @@
 
 #include "dct4.h"
 #include "../../gammu.h"
-#include "../../../common/phone/pfunc.h"
-#include "../../../common/phone/nokia/nfunc.h"
-#include "../../../common/phone/nokia/dct4s40/dct4func.h"
-#include "../../../common/misc/coding/coding.h"
-#include "../../../common/misc/locales.h"
+#include "../../../libgammu/phone/pfunc.h"
+#include "../../../libgammu/phone/nokia/nfunc.h"
+#include "../../../libgammu/phone/nokia/dct4s40/dct4func.h"
+#include "../../../libgammu/misc/coding/coding.h"
+#include "../../../helper/locales.h"
+
+#include "../../../helper/formats.h"
+#include "../../../helper/string.h"
 
 extern GSM_Reply_Function UserReplyFunctions4[];
 
@@ -61,7 +64,7 @@ static void CheckDCT4(void)
 	case ERR_OTHERCONNECTIONREQUIRED:
 		printf("%s\n", _("Can't do it with current phone protocol"));
 		GSM_TerminateConnection(gsm);
-		exit(-1);
+		Terminate(3);
 	default:
 		break;
 	}
@@ -75,7 +78,7 @@ static bool answer_yes2(const char *text)
 	while (1) {
 		printf(_("%s (yes/no) ? "),text);
 		len=GetLine(stdin, ans, 99);
-		if (len==-1) exit(-1);
+		if (len==-1) Terminate(3);
 		if (strcasecmp(ans, _("yes")) == 0) return true;
 		if (strcasecmp(ans, _("no")) == 0) return false;
 	}
@@ -419,8 +422,8 @@ static GSM_Error DCT4_ReplyVibra(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 #ifdef DEBUG
 	switch (msg.Buffer[3]) {
-		case 0x0D : dbgprintf("Vibra state set OK\n"); break;
-		case 0x0F : dbgprintf("Vibra power set OK\n"); break;
+		case 0x0D : smprintf(sm, "Vibra state set OK\n"); break;
+		case 0x0F : smprintf(sm, "Vibra power set OK\n"); break;
 	}
 #endif
 	return ERR_NONE;
@@ -440,7 +443,7 @@ static GSM_Error DCT4EnableVibra(GSM_StateMachine *sm, bool enable)
 void DCT4SetVibraLevel(int argc, char *argv[])
 {
 	GSM_DateTime	Date;
-	unsigned int	i,j;
+	int	i,j;
 	GSM_Error error;
 
 	/* Set vibra level */
@@ -588,9 +591,9 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
 
 	switch (msg.Buffer[3]) {
 	case 0x05:
-		dbgprintf("Part of voice record received\n");
+		smprintf(sm, "Part of voice record received\n");
 		if (msg.Length == 6) {
-			dbgprintf("Empty\n");
+			smprintf(sm, "Empty\n");
 			return ERR_EMPTY;
 		}
  		*sm->Phone.Data.VoiceRecord = 0;
@@ -601,20 +604,20 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
 		}
 		return ERR_NONE;
 	case 0x0D:
-		dbgprintf("Last part of voice record is %02x %02x\n",msg.Buffer[11],msg.Buffer[12]);
-		dbgprintf("Token is %02x\n",msg.Buffer[13]);
+		smprintf(sm, "Last part of voice record is %02x %02x\n",msg.Buffer[11],msg.Buffer[12]);
+		smprintf(sm, "Token is %02x\n",msg.Buffer[13]);
  		sm->Phone.Data.PhoneString[0] = msg.Buffer[11];
  		sm->Phone.Data.PhoneString[1] = msg.Buffer[12];
  		sm->Phone.Data.PhoneString[2] = msg.Buffer[13];
 		return ERR_NONE;
 	case 0x31:
-		dbgprintf("Names of voice records received\n");
+		smprintf(sm, "Names of voice records received\n");
 		j = 33;
 		for (i = 0; i < (size_t)msg.Buffer[9]; i++) {
 			memcpy(Buffer,msg.Buffer+(j+1),msg.Buffer[j]);
 			Buffer[msg.Buffer[j]] 	= 0;
 			Buffer[msg.Buffer[j]+1] = 0;
-			dbgprintf(SIZE_T_FORMAT ". \"%s\"\n",i+1,DecodeUnicodeString(Buffer));
+			smprintf(sm, "%ld. \"%s\"\n",(long)i+1,DecodeUnicodeString(Buffer));
  			if (i==*sm->Phone.Data.VoiceRecord) {
  				sprintf(sm->Phone.Data.PhoneString,"%s->wav",DecodeUnicodeString(Buffer));
 				return ERR_NONE;
@@ -691,13 +694,13 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 
 	gsm->Phone.Data.VoiceRecord 	= &Location;
 	gsm->Phone.Data.PhoneString 	= FileName;
-	dbgprintf("Getting voice record name\n");
+	smprintf(gsm, "Getting voice record name\n");
 	error=GSM_WaitFor (gsm, ReqNames, 14, 0x4A, 4, ID_User4);
 	Print_Error(error);
 
 	gsm->Phone.Data.PhoneString 	= Buffer;
 	ReqToken[7] 			= Location;
-	dbgprintf("Getting voice record token\n");
+	smprintf(gsm, "Getting voice record token\n");
 	error=GSM_WaitFor (gsm, ReqToken, 10, 0x23, 4, ID_User4);
 	Print_Error(error);
 	TokenLocation 			= Buffer[0] * 256 + Buffer[1];
@@ -717,7 +720,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	ReqGet[7]			= Location;
 	fprintf(stderr,"Getting voice record and saving to \"%s\": ",FileName);
 	while (1) {
-		dbgprintf("Getting next part of voice record\n");
+		smprintf(gsm, "Getting next part of voice record\n");
 		fprintf(stderr,".");
 		error=GSM_WaitFor (gsm, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
@@ -731,7 +734,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		ReqGet[12] = CurrentLocation % 256;
 		if (CurrentLocation+4 > TokenLocation) break;
 	}
-	dbgprintf("Getting first part in last sequence of voice record\n");
+	smprintf(gsm, "Getting first part in last sequence of voice record\n");
 	for (i=255;i>=0;i--) {
 		ReqGet[16] = i;
 		ReqGet[17] = Token;
@@ -745,7 +748,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		if (error != ERR_EMPTY) Print_Error(error);
 	}
 	while (1) {
-		dbgprintf("Getting next part of last sequence in voice record\n");
+		smprintf(gsm, "Getting next part of last sequence in voice record\n");
 		CurrentLocation += 4;
 		ReqGet[11] = CurrentLocation / 256;
 		ReqGet[12] = CurrentLocation % 256;
@@ -804,10 +807,10 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 
 	switch (msg.Buffer[3]) {
 	case 0x0D:
-		dbgprintf("Simlock info received\n");
-		dbgprintf("Config_Data: ");
+		smprintf(sm, "Simlock info received\n");
+		smprintf(sm, "Config_Data: ");
 		for (i=14;i<22;i++) {
-			dbgprintf("%02x",msg.Buffer[i]);
+			smprintf(sm, "%02x",msg.Buffer[i]);
 		}
 		sprintf(buff,"%02x%02x%02x",msg.Buffer[14],msg.Buffer[15],msg.Buffer[16]);
 		buff[6] = 0;
@@ -821,21 +824,21 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 				buff);
 		}
 
-		dbgprintf("\n");
-		dbgprintf("Profile_Bits: ");
+		smprintf(sm, "\n");
+		smprintf(sm, "Profile_Bits: ");
 		for (i=22;i<30;i++) {
-			dbgprintf("%02x",msg.Buffer[i]);
+			smprintf(sm, "%02x",msg.Buffer[i]);
 		}
-		dbgprintf("\n");
+		smprintf(sm, "\n");
 		return ERR_NONE;
 	case 0x13:
-		dbgprintf("Simlock info received\n");
+		smprintf(sm, "Simlock info received\n");
 		if (msg.Buffer[58] == 0x05 && msg.Buffer[59] == 0x02) {
-			dbgprintf("SIM_PATH: ");
+			smprintf(sm, "SIM_PATH: ");
 			for (i=44;i<52;i++) {
-				dbgprintf("%02x",msg.Buffer[i]);
+				smprintf(sm, "%02x",msg.Buffer[i]);
 			}
-			dbgprintf("\n");
+			smprintf(sm, "\n");
 			printf(LISTFORMAT, _("Simlock data"));
 			for (i=60;i<63;i++) {
 				printf("%02x",msg.Buffer[i]);
@@ -926,7 +929,7 @@ void DCT4GetT9(int argc, char *argv[])
 		Print_Error(error);
 		if (i==0) {
 			T9Dictionary = T9FullSize;
-			dbgprintf("T9 dictionary size is %i\n",T9Dictionary);
+			smprintf(gsm, "T9 dictionary size is %i\n",T9Dictionary);
 		}
 		i+=T9Size;
 	}
@@ -950,14 +953,14 @@ void DCT4SetLight(int argc, char *argv[])
 	} else if (strcasecmp(argv[2],"torch") == 0) {	type = N6510_LIGHT_TORCH;
 	} else {
 		printf(_("What lights should I enable (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	if (strcasecmp(argv[3],"on") == 0) { 		enable = true;
 	} else if (strcasecmp(argv[3],"off") == 0) {	enable = false;
 	} else {
 		printf(_("What should I do (\"%s\") ?\n"),argv[3]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	for (i=0;i<gsm->ConfigNum;i++) {
@@ -1149,14 +1152,14 @@ void DCT4TuneRadio(int argc, char *argv[])
 	if (error != ERR_NONE && error != ERR_EMPTY) {
 		printf("%s\n", _("Phone seems not to support radio"));
 		GSM_Terminate();
-		exit(-1);
+		Terminate(3);
 	}
 
 	error=GSM_WaitFor (gsm, Enable, 6, 0x3E, 4, ID_User3);
 	if (error == ERR_PERMISSION) {
 		printf("%s\n", _("Please connect headset. Required as antenna"));
 		GSM_Terminate();
-		exit(-1);
+		Terminate(3);
 	}
 	Print_Error(error);
 
@@ -1178,13 +1181,13 @@ void DCT4TuneRadio(int argc, char *argv[])
 				diff = RadioFreq - FMStation[j].Frequency;
 			}
 			if (diff <= 0.2) {
-				dbgprintf("diff is %f\n",diff);
+				smprintf(gsm, "diff is %f\n",diff);
 				found = true;
 				break;
 			}
 		}
 		if (!found) {
-			dbgprintf("Adding %f, num %i\n",RadioFreq,num);
+			smprintf(gsm, "Adding %f, num %i\n",RadioFreq,num);
 			FMStation[num].Frequency = RadioFreq;
 			CopyUnicodeString(FMStation[num].StationName,RadioName);
 			num++;
@@ -1355,7 +1358,7 @@ static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMac
 		switch (msg.Buffer[pos+4]) {
 		case N7110_PBK_SIM_SPEEDDIAL	: printf(_(" (Speed dial on SIM)")); 			break;
 		case N7110_PBK_NAME	    	: printf(_(" (Text: name (always the only one)"));		break;
-		case N7110_PBK_EMAIL	    	: printf(_(" (Text: email adress)"));			break;
+		case N7110_PBK_EMAIL	    	: printf(_(" (Text: email address)"));			break;
 		case N7110_PBK_POSTAL	    	: printf(_(" (Text: postal address)"));			break;
 		case N7110_PBK_NOTE	    	: printf(_(" (Text: note)"));				break;
 		case N7110_PBK_NUMBER 	    	: printf(_(" (Phone number)"));				break;
@@ -1429,7 +1432,7 @@ void DCT4GetPBKFeatures(int argc, char *argv[])
 	}
 
 	req[4] = NOKIA_GetMemoryType(gsm, MemoryType,N71_65_MEMORY_TYPES);
-	if (req[4]==0xff) exit(-1);
+	if (req[4]==0xff) Terminate(3);
 
 	GSM_Init(true);
 
