@@ -4,153 +4,17 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "../common/misc/locales.h"
+#include "../helper/locales.h"
 
 #include "common.h"
 #include "memory.h"
-#include "formats.h"
+#include "../helper/formats.h"
+#include "../helper/memory-display.h"
+#include "../helper/printing.h"
+#include "../helper/string.h"
 
 #define GAMMU_CALLER_GROUPS	20
 
-static GSM_Bitmap		caller[GAMMU_CALLER_GROUPS];
-static GSM_AllRingtonesInfo 	Info = {0, NULL};
-static bool			callerinit[GAMMU_CALLER_GROUPS] = {false, false, false, false, false};
-static bool			ringinit = false;
-
-void PrintMemorySubEntry(GSM_SubMemoryEntry *entry)
-{
-	GSM_Category	Category;
-	int		z;
-	GSM_Error error;
-
-	switch (entry->EntryType) {
-	case PBK_CallLength:
-		printf(_("Call length      : %02i:%02i:%02i\n"),entry->CallLength/(60*60),entry->CallLength/60,entry->CallLength%60);
-		return;
-	case PBK_Date:
-		printf(LISTFORMAT "%s\n", _("Date and time"),OSDateTime(entry->Date,false));
-		return;
-	case PBK_LastModified:
-		printf(LISTFORMAT "%s\n", _("Last modified"), OSDateTime(entry->Date,false));
-		return;
-	case PBK_Category:
-		if (entry->Number == -1) {
-			printf(LISTFORMAT "\"%s\"\n", _("Category"), DecodeUnicodeConsole(entry->Text));
-		} else {
-			Category.Location = entry->Number;
-			Category.Type = Category_Phonebook;
-			error=GSM_GetCategory(gsm, &Category);
-			if (error == ERR_NONE) {
-				printf(LISTFORMAT "\"%s\" (%i)\n", _("Category"), DecodeUnicodeConsole(Category.Name), entry->Number);
-			} else {
-				printf(LISTFORMAT "%i\n", _("Category"), entry->Number);
-			}
-		}
-		return;
-	case PBK_Private:
-		printf(LISTFORMAT "%s\n", _("Private"), entry->Number == 1 ? "Yes" : "No");
-		return;
-	case PBK_Caller_Group       :
-		if (entry->Number >= GAMMU_CALLER_GROUPS) {
-			printf(LISTFORMAT "\"%d\"\n", _("Caller group"),entry->Number);
-			fprintf(stderr, "%s\n", _("Caller group number too high, please increase buffer in sources!"));
-			break;
-		}
-		if (!callerinit[entry->Number-1]) {
-			caller[entry->Number-1].Type	    = GSM_CallerGroupLogo;
-			caller[entry->Number-1].Location = entry->Number;
-			error=GSM_GetBitmap(gsm,&caller[entry->Number-1]);
-			Print_Error(error);
-			if (caller[entry->Number-1].DefaultName) {
-				NOKIA_GetDefaultCallerGroupName(&caller[entry->Number-1]);
-			}
-			callerinit[entry->Number-1]=true;
-		}
-		printf(LISTFORMAT "\"%s\"\n", _("Caller group"),DecodeUnicodeConsole(caller[entry->Number-1].Text));
-		return;
-	case PBK_RingtoneID	     :
-		if (!ringinit) {
-			error=GSM_GetRingtonesInfo(gsm,&Info);
-			if (error != ERR_NOTSUPPORTED) Print_Error(error);
-			if (error == ERR_NONE) ringinit = true;
-		}
-		if (ringinit) {
-			for (z=0;z<Info.Number;z++) {
-				if (Info.Ringtone[z].ID == entry->Number) {
-					printf(LISTFORMAT "\"%s\"\n", _("Ringtone"),DecodeUnicodeConsole(Info.Ringtone[z].Name));
-					break;
-				}
-			}
-		} else {
-			printf(LISTFORMAT "%i\n", _("Ringtone ID"),entry->Number);
-		}
-		return;
-	case PBK_Text_UserID:
-		printf(LISTFORMAT "%s\n", _("User ID"),DecodeUnicodeString(entry->Text));
-		return;
-	case PBK_Text_PictureName:
-		printf(LISTFORMAT "%s\n", _("Picture name"),DecodeUnicodeString(entry->Text));
-		return;
-	case PBK_PictureID	     :
-		printf(LISTFORMAT "0x%x\n", _("Picture ID"),entry->Number);
-		return;
-	case PBK_Photo:
-		printf(LISTFORMAT "%s\n", _("Photo"), _("Displaying not supported"));
-		return;
-	case PBK_Number_Messaging   : printf(LISTFORMAT, _("Favorite messaging number")); break;
-	case PBK_Number_General     : printf(LISTFORMAT, _("General number")); break;
-	case PBK_Number_Mobile      : printf(LISTFORMAT, _("Mobile number")); break;
-	case PBK_Number_Work        : printf(LISTFORMAT, _("Work number")); break;
-	case PBK_Number_Fax         : printf(LISTFORMAT, _("Fax number")); break;
-	case PBK_Number_Home        : printf(LISTFORMAT, _("Home number")); break;
-	case PBK_Number_Pager       : printf(LISTFORMAT, _("Pager number")); break;
-	case PBK_Number_Other       : printf(LISTFORMAT, _("Other number")); break;
-	case PBK_Text_Note          : printf(LISTFORMAT, _("Text")); break;
-	case PBK_Text_Postal        : printf(LISTFORMAT, _("Snail address")); break;
-	case PBK_Text_WorkPostal    : printf(LISTFORMAT, _("Work snail address")); break;
-	case PBK_Text_Email         : printf(LISTFORMAT, _("Email address 1")); break;
-	case PBK_Text_Email2        : printf(LISTFORMAT, _("Email address 2")); break;
-	case PBK_Text_URL           : printf(LISTFORMAT, _("URL address")); break;
-	case PBK_Text_LUID          : printf(LISTFORMAT, _("LUID")); break;
-	case PBK_Text_Name          : printf(LISTFORMAT, _("Name")); break;
-	case PBK_Text_LastName      : printf(LISTFORMAT, _("Last name")); break;
-	case PBK_Text_FirstName     : printf(LISTFORMAT, _("First name")); break;
-	case PBK_Text_FormalName     : printf(LISTFORMAT, _("Formal name")); break;
-	case PBK_Text_NickName     : printf(LISTFORMAT, _("Nick name")); break;
-	case PBK_Text_Company       : printf(LISTFORMAT, _("Company")); break;
-	case PBK_Text_JobTitle      : printf(LISTFORMAT, _("Job title")); break;
-	case PBK_Text_StreetAddress : printf(LISTFORMAT, _("Street address")); break;
-	case PBK_Text_City          : printf(LISTFORMAT, _("City")); break;
-	case PBK_Text_State         : printf(LISTFORMAT, _("State")); break;
-	case PBK_Text_Zip           : printf(LISTFORMAT, _("Zip code")); break;
-	case PBK_Text_Country       : printf(LISTFORMAT, _("Country")); break;
-	case PBK_Text_WorkStreetAddress : printf(LISTFORMAT, _("Work street address")); break;
-	case PBK_Text_WorkCity      : printf(LISTFORMAT, _("Work city")); break;
-	case PBK_Text_WorkState     : printf(LISTFORMAT, _("Work state")); break;
-	case PBK_Text_WorkZip       : printf(LISTFORMAT, _("Work zip code")); break;
-	case PBK_Text_WorkCountry   : printf(LISTFORMAT, _("Work country")); break;
-	case PBK_Text_Custom1       : printf(LISTFORMAT, _("Custom text 1")); break;
-	case PBK_Text_Custom2       : printf(LISTFORMAT, _("Custom text 2")); break;
-	case PBK_Text_Custom3       : printf(LISTFORMAT, _("Custom text 3")); break;
-	case PBK_Text_Custom4       : printf(LISTFORMAT, _("Custom text 4")); break;
-	case PBK_PushToTalkID       : printf(LISTFORMAT, _("Push to talk ID")); break;
-#ifndef CHECK_CASES
-	default:
-		printf("%s\n", _("unknown field type"));
-		return;
-#endif
-	}
-	printf("\"%s\"\n", DecodeUnicodeConsole(entry->Text));
-}
-
-void PrintMemoryEntry(GSM_MemoryEntry *entry)
-{
-	int i;
-
-	for (i=0;i<entry->EntriesNum;i++) PrintMemorySubEntry(&entry->Entries[i]);
-	printf("\n");
-}
-
 void GetAllMemory(int argc UNUSED, char *argv[])
 {
 	GSM_MemoryEntry		Entry;
@@ -173,14 +37,16 @@ void GetAllMemory(int argc UNUSED, char *argv[])
 	while (!gshutdown) {
 		error = GSM_GetNextMemory(gsm, &Entry, start);
 		if (error == ERR_EMPTY) break;
-		if (error != ERR_NONE && Info.Ringtone) free(Info.Ringtone);
+		if (error != ERR_NONE && ringtones_info.Ringtone) free(ringtones_info.Ringtone);
 		Print_Error(error);
 		printf(_("Memory %s, Location %i\n"),argv[2],Entry.Location);
-		PrintMemoryEntry(&Entry);
+		error = PrintMemoryEntry(&Entry, gsm);
+		GSM_FreeMemoryEntry(&Entry);
+		Print_Error(error);
  		start = false;
 	}
 
- 	if (Info.Ringtone) free(Info.Ringtone);
+ 	if (ringtones_info.Ringtone) free(ringtones_info.Ringtone);
 
 	GSM_Terminate();
 }
@@ -233,7 +99,7 @@ void GetMemory(int argc, char *argv[])
 
 		error=GSM_GetMemory(gsm, &entry);
 		if (error != ERR_EMPTY) {
-			if (Info.Ringtone) free(Info.Ringtone);
+			if (ringtones_info.Ringtone) free(ringtones_info.Ringtone);
 			Print_Error(error);
 		}
 
@@ -246,13 +112,15 @@ void GetMemory(int argc, char *argv[])
 		} else {
 			fillednum++;
 			if (!empty) printf(_("Memory %s, Location %i\n"),argv[2],j);
-			PrintMemoryEntry(&entry);
+			error = PrintMemoryEntry(&entry, gsm);
+			GSM_FreeMemoryEntry(&entry);
+			Print_Error(error);
 		}
 	}
 
 	printf(_("%i entries empty, %i entries filled\n"),emptynum,fillednum);
 
- 	if (Info.Ringtone) free(Info.Ringtone);
+ 	if (ringtones_info.Ringtone) free(ringtones_info.Ringtone);
 
 	GSM_Terminate();
 }
@@ -318,6 +186,7 @@ void DeleteAllMemory(int argc, char *argv[])
 void SearchOneEntry(GSM_MemoryEntry *Entry, const unsigned char *Text)
 {
 	int i;
+	GSM_Error error;
 
 	for (i=0;i<Entry->EntriesNum;i++) {
 		switch (Entry->Entries[i].EntryType) {
@@ -360,14 +229,15 @@ void SearchOneEntry(GSM_MemoryEntry *Entry, const unsigned char *Text)
 			case PBK_Text_UserID:
 			case PBK_Text_PictureName:
 			case PBK_PushToTalkID:
-			case PBK_Caller_Group       :
 				if (mywstrstr(Entry->Entries[i].Text, Text) != NULL) {
 					fprintf(stderr,"\n");
 					printf(_("Memory %s, Location %i\n"),GSM_MemoryTypeToString(Entry->MemoryType),Entry->Location);
-					PrintMemoryEntry(Entry);
+					error = PrintMemoryEntry(Entry, gsm);
+					Print_Error(error);
 					return;
 				}
 				break;
+			case PBK_Caller_Group:
 			case PBK_Date:
 			case PBK_Category:
 			case PBK_Private:
@@ -389,25 +259,25 @@ void SearchOneMemory(GSM_MemoryType MemoryType, const char *Title, const unsigne
 	GSM_MemoryEntry		Entry;
 	GSM_MemoryStatus	Status;
 	int			i = 0, l = 1;
-	bool			start = true;
 	GSM_Error error;
 
 	Status.MemoryType = MemoryType;
 	Entry.MemoryType  = MemoryType;
+	Entry.Location = 0;
 
 	if (GSM_GetMemoryStatus(gsm, &Status) == ERR_NONE) {
 		fprintf(stderr,"%c%s: %i%%", 13, Title, (i+1)*100/(Status.MemoryUsed+1));
 		error = GSM_GetNextMemory(gsm, &Entry, true);
 		if (error != ERR_NOTSUPPORTED && error != ERR_NOTIMPLEMENTED) {
 			while (i < Status.MemoryUsed) {
-				if (gshutdown) return;
-				i++;
-				fprintf(stderr,"\r%s: %i%%", Title, (i+1)*100/(Status.MemoryUsed+1));
-				error = GSM_GetNextMemory(gsm, &Entry, start);
 				if (error == ERR_EMPTY) break;
 				Print_Error(error);
 				SearchOneEntry(&Entry, Text);
-				start = false;
+				GSM_FreeMemoryEntry(&Entry);
+				if (gshutdown) return;
+				i++;
+				fprintf(stderr,"\r%s: %i%%", Title, (i+1)*100/(Status.MemoryUsed+1));
+				error = GSM_GetNextMemory(gsm, &Entry, false);
 			}
 		} else {
 			while (i < Status.MemoryUsed) {
@@ -417,6 +287,7 @@ void SearchOneMemory(GSM_MemoryType MemoryType, const char *Title, const unsigne
 					Print_Error(error);
 					i++;
 					SearchOneEntry(&Entry, Text);
+					GSM_FreeMemoryEntry(&Entry);
 				}
 				fprintf(stderr,"%c%s: %i%%", 13, Title, (i+1)*100/(Status.MemoryUsed+1));
 				l++;
@@ -473,9 +344,11 @@ void ListMemoryCategoryEntries(int Category)
 		for (j=0;j<Entry.EntriesNum;j++) {
 			if (Entry.Entries[j].EntryType == PBK_Category && Entry.Entries[j].Number == Category) {
 				printf(_("Memory %s, Location %i\n"),GSM_MemoryTypeToString(Entry.MemoryType),Entry.Location);
-				PrintMemoryEntry(&Entry);
+				error = PrintMemoryEntry(&Entry, gsm);
+				Print_Error(error);
 			}
 		}
+		GSM_FreeMemoryEntry(&Entry);
  		start = false;
 	}
 }
