@@ -1,13 +1,11 @@
 #include <gammu.h>
 
 extern volatile bool wasincomingsms;
-extern void IncomingSMS(GSM_StateMachine * s, GSM_SMSMessage sms);
-extern void IncomingCB(GSM_StateMachine * s, GSM_CBMessage CB);
-extern void IncomingUSSD(GSM_StateMachine * s, GSM_USSDMessage ussd);
-extern void IncomingUSSD2(GSM_StateMachine * s, GSM_USSDMessage ussd);
+extern void IncomingSMS(GSM_StateMachine * s, GSM_SMSMessage sms, void *user_data);
+extern void IncomingCB(GSM_StateMachine * s, GSM_CBMessage CB, void *user_data);
+extern void IncomingUSSD(GSM_StateMachine * s, GSM_USSDMessage ussd, void *user_data);
+extern void IncomingUSSD2(GSM_StateMachine * s, GSM_USSDMessage ussd, void *user_data);
 extern void DisplayIncomingSMS(void);
-extern void DisplayMultiSMSInfo(GSM_MultiSMSMessage *sms, bool eachsms, bool ems,
-				const GSM_Backup * Info);
 extern void GetSMSC(int argc, char *argv[]);
 extern void GetSMS(int argc, char *argv[]);
 extern void DeleteSMS(int argc, char *argv[]);
