@@ -1,8 +1,7 @@
 #define _GNU_SOURCE /* For strcasestr */
 #include <string.h>
 
-#include "../common/misc/locales.h"
-#include "../common/misc/coding/coding.h"
+#include "../helper/locales.h"
 
 #include <gammu.h>
 #include <stdlib.h>
@@ -18,9 +17,12 @@
 
 #include "nokia.h"
 #include "common.h"
-#include "formats.h"
 #include "files.h"
 
+#include "../helper/formats.h"
+#include "../helper/printing.h"
+#include "../helper/string.h"
+
 #if defined(GSM_ENABLE_NOKIA_DCT3) || defined(GSM_ENABLE_NOKIA_DCT4)
 void NokiaComposer(int argc UNUSED, char *argv[])
 {
@@ -39,7 +41,7 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 
 	if (ringtone.Format != RING_NOTETONE) {
 		printf("%s\n", _("It can be RTTL ringtone only used with this option"));
-		exit(-1);
+		Terminate(3);
 	}
 
 	started = false;
@@ -49,16 +51,19 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 			Note = &ringtone.NoteTone.Commands[i].Note;
 			if (!started) {
 				if (Note->Note != Note_Pause) {
-					printf(_("Ringtone \"%s\" (tempo = %i Beats Per Minute)\n\n"),DecodeUnicodeConsole(ringtone.Name),GSM_RTTLGetTempo(Note->Tempo));
+					printf(_("Ringtone \"%s\" (tempo = %i Beats Per Minute)"),DecodeUnicodeConsole(ringtone.Name),GSM_RTTLGetTempo(Note->Tempo));
+					printf("\n\n");
 					started = true;
 				}
 			}
 			if (started) j++;
 		}
 	}
-    	if (j>50) printf_warn(_("length=%i notes, but you will enter only first 50 tones."),j);
+    	if (j>50) {
+		printf_warn(_("length=%i notes, but you will enter only first 50 tones."), j);
+	}
 
-	printf(_("\n\nThis ringtone in Nokia Composer in phone should look: "));
+	printf("\n\n%s ", _("This ringtone in Nokia Composer in phone should look:"));
 	started = false;
 	for (i=0;i<ringtone.NoteTone.NrCommands;i++) {
 		if (ringtone.NoteTone.Commands[i].Type == RING_Note) {
@@ -97,7 +102,7 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 		}
 	}
 
-	printf(_("\n\nTo enter it please press: "));
+	printf("\n\n%s ", _("To enter it please press:"));
 	started = false;
 	for (i=0;i<ringtone.NoteTone.NrCommands;i++) {
 		if (ringtone.NoteTone.Commands[i].Type == RING_Note) {
@@ -143,7 +148,7 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 					case Duration_1_8  : Duration = 16;	break;
 					case Duration_1_16 : Duration = 8;	break;
 					case Duration_1_32 : Duration = 4;	break;
-					default		   : dbgprintf("error\n");break;
+					default		   : fprintf(stderr, "error\n");break;
 				}
 				if (Duration > DefNoteDuration) {
 		        		while (DefNoteDuration != Duration) {
@@ -163,6 +168,7 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 	}
 
 	printf("\n");
+	fflush(stdout);
 }
 
 void NokiaSecurityCode(int argc, char *argv[])
@@ -305,7 +311,7 @@ void NokiaAddPlayLists2(unsigned char *ID,unsigned char *Name,unsigned char *IDF
 			CopyUnicodeString(Files3.ID_FullName,IDFolder);
 	        	CopyUnicodeString(Buffer2,Name);
 			if (i!=0) {
-				sprintf(Buffer, SIZE_T_FORMAT, i);
+				sprintf(Buffer, "%ld", (long)i);
 		        	EncodeUnicode(Buffer2+UnicodeLength(Buffer2)*2,Buffer,strlen(Buffer));
 			}
 	        	EncodeUnicode(Buffer2+UnicodeLength(Buffer2)*2,".m3u",4);
@@ -362,7 +368,7 @@ void NokiaAddPlayLists2(unsigned char *ID,unsigned char *Name,unsigned char *IDF
 		}
 
 		/* adding new playlist file */
-		sprintf(Buffer2,"  Writing %s: ",DecodeUnicodeString(Files2.Name));
+		sprintf(Buffer2, _("Writing file %s:"), DecodeUnicodeString(Files2.Name));
 		AddOneFile(&Files2, Buffer2, false);
 
 		/* cleaning buffers */
@@ -419,7 +425,7 @@ void NokiaAddPlayLists(int argc UNUSED, char *argv[] UNUSED)
 	if (error == ERR_FILENOTEXIST) {
 		printf("%s\n", _("Your phone model is not supported. Please report"));
 		GSM_Terminate();
-		exit(-1);
+		Terminate(3);
 	} else if (error != ERR_EMPTY) {
 	    	Print_Error(error);
 	}
@@ -528,7 +534,7 @@ void NokiaAddFile(int argc, char *argv[])
 	}
 	if (!Found) {
 		printf(_("What folder type (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	if (strcasecmp(argv[2],"Application") == 0 || strcasecmp(argv[2],"Game") == 0) {
@@ -563,7 +569,7 @@ void NokiaAddFile(int argc, char *argv[])
 		if (!Found) {
 			printf("%s\n", _("Folder not found. Probably function not supported!"));
 			GSM_Terminate();
-			exit(-1);
+			Terminate(3);
 		}
 	} else if (GSM_IsPhoneFeatureAvailable(GSM_GetModelInfo(gsm), F_FILES2)) {
 		i = 0;
@@ -591,16 +597,16 @@ void NokiaAddFile(int argc, char *argv[])
 		    	Print_Error(error);
 
 			if (Files.Folder) {
-				dbgprintf("folder %s level %i\n",DecodeUnicodeConsole(Files.Name),Files.Level);
+				smprintf(gsm, "folder %s level %i\n",DecodeUnicodeConsole(Files.Name),Files.Level);
 				Found 	= false;
 				i 	= 0;
 				while (Folder[i].parameter[0] != 0) {
 					EncodeUnicode(buffer,Folder[i].folder,strlen(Folder[i].folder));
-					dbgprintf("comparing \"%s\" \"%s\" \"%s\"\n",GSM_GetModelInfo(gsm)->model,DecodeUnicodeString(Files.ID_FullName),Folder[i].level);
+					smprintf(gsm, "comparing \"%s\" \"%s\" \"%s\"\n",GSM_GetModelInfo(gsm)->model,DecodeUnicodeString(Files.ID_FullName),Folder[i].level);
 					if (strcasecmp(argv[2],Folder[i].parameter) == 0  &&
 					    mywstrncasecmp(Files.Name,buffer,0) &&
 					    Files.Level == atoi(Folder[i].level)) {
-						dbgprintf("found 1\n");
+						smprintf(gsm, "found 1\n");
 						Found = true;
 						break;
 					}
@@ -608,18 +614,18 @@ void NokiaAddFile(int argc, char *argv[])
 					    !strcmp(DecodeUnicodeString(Files.ID_FullName),Folder[i].folder) &&
 					    Folder[i].level[0] == 0x00) {
 						Found = true;
-						dbgprintf("found 2\n");
+						smprintf(gsm, "found 2\n");
 						break;
 					}
 					if (Folder[i].folder[0]>='0'&&Folder[i].folder[0] <='9') {
 						DecodeHexUnicode (buffer, Folder[i].folder,strlen(Folder[i].folder));
-						dbgprintf("comparing \"%s\"",DecodeUnicodeString(buffer));
-						dbgprintf("and \"%s\"\n",DecodeUnicodeString(Files.Name));
+						smprintf(gsm, "comparing \"%s\"",DecodeUnicodeString(buffer));
+						smprintf(gsm, "and \"%s\"\n",DecodeUnicodeString(Files.Name));
 						if (strcasecmp(argv[2],Folder[i].parameter) == 0  &&
 						    mywstrncasecmp(Files.Name,buffer,0) &&
 						    Files.Level == atoi(Folder[i].level)) {
 							Found = true;
-							dbgprintf("found 3\n");
+							smprintf(gsm, "found 3\n");
 							break;
 						}
 					}
@@ -632,11 +638,12 @@ void NokiaAddFile(int argc, char *argv[])
 			Start = false;
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 	if (!Found) {
 		printf("%s\n", _("Folder not found. Probably function not supported!"));
 		GSM_Terminate();
-		exit(-1);
+		Terminate(3);
 	}
 	File.Buffer 	= NULL;
 	File.Protected  = false;
@@ -676,7 +683,7 @@ void NokiaAddFile(int argc, char *argv[])
 			}
 		}
 		if (Size != Size2) {
-			fprintf(stderr, "%s\n", _("INFO: declared JAR file size is different than real. Fixed by Gammu"));
+			printf_info("%s\n", _("Declared JAR file size is different than real. Fixed by Gammu."));
 			for (i=0;i<File.Used;i++) {
 				if (strncasecmp(File.Buffer+i,"MIDlet-Jar-Size: ",17) == 0) {
 					break;
@@ -700,7 +707,7 @@ void NokiaAddFile(int argc, char *argv[])
 			File.Used = File.Used - (i - Pos);
 			File.Buffer = realloc(File.Buffer,File.Used);
 		} else if (Size == -1) {
-			fprintf(stderr, "%s\n", _("INFO: no JAR size info in JAD file. Added by Gammu"));
+			printf_info("%s\n", _("No JAR size info in JAD file. Added by Gammu."));
 		}
 		if (Size != Size2) {
 			sprintf(buffer,"\nMIDlet-Jar-Size: %i",Size2);
@@ -719,7 +726,7 @@ void NokiaAddFile(int argc, char *argv[])
 			while (JAR[i] != '/') i--;
 			strcpy(buffer,JAR+i+1);
 			strcpy(JAR,buffer);
-			dbgprintf("New file name is \"%s\"\n",JAR);
+			smprintf(gsm, "New file name is \"%s\"\n",JAR);
 		}
 
 		/* Changing all #13 or #10 to #13#10 in JAD */
@@ -762,7 +769,7 @@ void NokiaAddFile(int argc, char *argv[])
 			CopyUnicodeString(File.ID_FullName,Files.ID_FullName);
 			error = GSM_AddFolder(gsm,&File);
 			if (Overwrite && (error == ERR_FILEALREADYEXIST)) {
-				fprintf(stderr, "%s\n", _("INFO: Application already exist. Deleting by Gammu"));
+				printf_info("%s\n", _("Application already exists. Deleting by Gammu."));
 
 				Start = true;
 				CopyUnicodeString(File2.ID_FullName,Files.ID_FullName);
@@ -814,7 +821,7 @@ void NokiaAddFile(int argc, char *argv[])
 				Start = true;
 				CopyUnicodeString(File2.ID_FullName,Files.ID_FullName);
 
-				fprintf(stderr, "%s\n", _("INFO: Application already exist. Deleting by Gammu"));
+				printf_info("%s\n", _("Application already exists. Deleting by Gammu."));
 
 				while (true) {
 					error = GSM_GetFolderListing(gsm,&File2,Start);
@@ -850,8 +857,8 @@ void NokiaAddFile(int argc, char *argv[])
 		EncodeUnicode(File.Name,buffer,strlen(buffer));
 		File.Type 	   = GSM_File_Other;
 		File.ModifiedEmpty = true;
-		dbgprintf("file id is \"%s\"\n",DecodeUnicodeString(File.ID_FullName));
-		AddOneFile(&File, "Writing JAD file: ", false);
+		smprintf(gsm, "file id is \"%s\"\n",DecodeUnicodeString(File.ID_FullName));
+		AddOneFile(&File, _("Writing JAD file:"), false);
 
 		if (argc > 4) {
 			for (j = 4; j < argc; j++) {
@@ -870,7 +877,7 @@ void NokiaAddFile(int argc, char *argv[])
 		EncodeUnicode(File.Name,buffer,strlen(buffer));
 		File.Type 	   = GSM_File_Java_JAR;
 		File.ModifiedEmpty = true;
-		AddOneFile(&File, "Writing JAR file: ", false);
+		AddOneFile(&File, _("Writing JAR file:"), false);
 		free(File.Buffer);
 		GSM_Terminate();
 		return;
@@ -916,7 +923,7 @@ void NokiaAddFile(int argc, char *argv[])
 						continue;
 					}
 					printf(_("Parameter \"%s\" unknown\n"),argv[j]);
-					exit(-1);
+					Terminate(3);
 				case 1:
 					strcpy(buffer,argv[j]);
 					nextlong = 0;
@@ -925,7 +932,7 @@ void NokiaAddFile(int argc, char *argv[])
 			}
 			if (nextlong!=0) {
 				printf_err("%s\n", _("Parameter missing!"));
-				exit(-1);
+				Terminate(3);
 			}
 		}
 	}
@@ -945,7 +952,7 @@ void NokiaAddFile(int argc, char *argv[])
 
 	GSM_IdentifyFileFormat(&File);
 
-	AddOneFile(&File, "Writing file: ", false);
+	AddOneFile(&File, _("Writing file:"), false);
 	free(File.Buffer);
 	GSM_Terminate();
 }
