@@ -68,7 +68,7 @@ extern void GSM_InitLocales(const char *path);
 #  if !defined(HAVE_UNISTD_H) || defined(__MINGW32__)
 #    define sleep(x) Sleep((x) * 1000)
 #    define usleep(x) Sleep(((x) < 1000) ? 1 : ((x) / 1000))
-#  endif /* HAVE_UNISTD_H */
+#  endif			/* HAVE_UNISTD_H */
 #endif
 
 /* Easy check for GCC */
@@ -108,17 +108,6 @@ extern void GSM_InitLocales(const char *path);
 #define INLINE
 #endif
 
-/* Printf string for size_t */
-#ifdef WIN32
-#define SIZE_T_FORMAT "%Id"
-#define SIZE_T_HEX_FORMAT "%02IX"
-#define SIZE_T_HEX4_FORMAT "%04IX"
-#else
-#define SIZE_T_FORMAT "%zd"
-#define SIZE_T_HEX_FORMAT "%02zX"
-#define SIZE_T_HEX4_FORMAT "%04zX"
-#endif
-
 /* Working snprintf on MSVC */
 #ifdef _MSC_VER
 #define snprintf _snprintf
@@ -164,7 +153,8 @@ void EncodeHexBin(unsigned char *dest, const unsigned char *src, int len);
  *
  * \return True if latest_version > current_version.
  */
-bool GSM_IsNewerVersion(const char *latest_version, const char *current_version);
+bool GSM_IsNewerVersion(const char *latest_version,
+			const char *current_version);
 #endif
 
 /* Editor configuration
