@@ -16,6 +16,14 @@
 #include <gammu-error.h>
 #include <gammu-inifile.h>
 
+/**
+ * Callback function for logging.
+ *
+ * \param text Text to be printed, \n will be also sent (as a separate
+ * message).
+ */
+typedef void (*GSM_Log_Function) (const char *text, void *data);
+
 /**
  * Private structure holding information about phone connection. Should
  * be allocated by \ref GSM_AllocStateMachine and freed by
@@ -61,10 +69,6 @@ typedef struct {
 	 * Name of debug file
 	 */
 	char *DebugFile;
-	/**
-	 * Name of localisation file
-	 */
-	char *Localize;
 	/**
 	 * Display something during start ?
 	 */
@@ -96,7 +100,7 @@ typedef struct {
 	/**
 	 * Phone features override.
 	 */
-	Feature PhoneFeatures[GSM_MAX_PHONE_FEATURES + 1];
+	GSM_Feature PhoneFeatures[GSM_MAX_PHONE_FEATURES + 1];
 } GSM_Config;
 
 /**
@@ -123,9 +127,27 @@ typedef enum {
 	GCT_BLUEFBUS2,
 	GCT_BLUEAT,
 	GCT_BLUEPHONET,
-	GCT_BLUEOBEX
+	GCT_BLUEOBEX,
+	GCT_FBUS2USB,
+	GCT_NONE
 } GSM_ConnectionType;
 
+/**
+ * Initiates connection with custom logging callback.
+ *
+ * \ingroup StateMachine
+ *
+ * \param s State machine data
+ * \param ReplyNum Number of replies to await (usually 3).
+ * \param log_function Logging function, see GSM_SetDebugFunction.
+ * \param user_data User data for logging function, see GSM_SetDebugFunction.
+ * \return Error code
+ * \see GSM_SetDebugFunction
+ */
+GSM_Error GSM_InitConnection_Log(GSM_StateMachine * s, int ReplyNum,
+				 GSM_Log_Function log_function,
+				 void *user_data);
+
 /**
  * Initiates connection.
  *
@@ -177,11 +199,14 @@ bool GSM_IsConnected(GSM_StateMachine * s);
  * in home and last fallback is in current driectory.
  *
  * \param result Ini file representation
+ * \param force_config Forcing of custom path instead of autodetected
+ * one (if NULL, autodetection is performed).
+ *
  * \return Error code
  *
  * \ingroup StateMachine
  */
-GSM_Error GSM_FindGammuRC(INI_Section ** result);
+GSM_Error GSM_FindGammuRC(INI_Section ** result, const char *force_config);
 
 /**
  * Processes gammu configuration.
