@@ -3,10 +3,13 @@
 #include <gammu.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include "../common/protocol/protocol.h" /* Needed for GSM_Protocol_Message */
-#include "../common/gsmstate.h" /* Needed for state machine internals */
+#include "../libgammu/protocol/protocol.h" /* Needed for GSM_Protocol_Message */
+#include "../libgammu/gsmstate.h" /* Needed for state machine internals */
+#include "../libgammu/gsmphones.h" /* Phone data */
 
-#include "sms-printing.h"
+#include "../helper/message-display.h"
+
+#include "common.h"
 
 unsigned char data[] = {
 	0x01, 0x6A, 0x00, 0x03, 0x00, 0x07, 0x01, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x02, 0x31,
@@ -32,10 +35,8 @@ int main(int argc UNUSED, char **argv UNUSED)
 
 	/* Allocates state machine */
 	s = GSM_AllocStateMachine();
-	if (s == NULL) {
-		printf("Could not allocate state machine!\n");
-		return 1;
-	}
+	test_result (s != NULL);
+
 	debug_info = GSM_GetDebug(s);
 	GSM_SetDebugGlobal(true, debug_info);
 	GSM_SetDebugFileDescriptor(stderr, false, debug_info);
@@ -55,14 +56,15 @@ int main(int argc UNUSED, char **argv UNUSED)
 	error = N6510_ReplyGetSMSMessage(msg, s);
 
 	/* Display message */
-	DisplayTestSMS(sms);
+	DisplayMultiSMSInfo(&sms, false, true, NULL, NULL);
+	DisplayMultiSMSInfo(&sms, true, true, NULL, NULL);
 
 	/* Free state machine */
 	GSM_FreeStateMachine(s);
 
-	printf("%s\n", GSM_ErrorString(error));
+	gammu_test_result(error, "N6510_ReplyGetSMSMessage");
 
-	return (error == ERR_NONE) ? 0 : 1;
+	return 0;
 }
 
 /* Editor configuration
